--[[
    Baldrick by LordFarlander
    Version 1.5.95
    $Revision: 94 $

    Based loosely on Minipet.
]]--

--[[
Copyright (c) 2008, LordFarlander
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

    * Redistributions of source code must retain the above copyright notice,
      this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright notice,
      this list of conditions and the following disclaimer in the documentation
      and/or other materials provided with the distribution.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
]]--

local L = LibStub( "AceLocale-3.0" ):GetLocale( "Baldrick" );

_G["BINDING_NAME_CLICK Baldrick_Button:LeftButton"] = L["Use Quest Item"];
BINDING_HEADER_BALDRICKHEADER = L["Baldrick"];

local LibLordFarlander = LibStub( "LibLordFarlander-2.0", true );
if( not LibLordFarlander ) then
    error( L["%s requires %s"]:format( L["Baldrick"], "LibLordFarlander-2.0" ) );
    return;
end--if

if( not LibStub( "LibLordFarlander-UI-2.0", true ) ) then
    error( L["%s requires %s"]:format( L["Baldrick"], "LibLordFarlander-UI-2.0" ) );
    return;
end--if

local quixote = LibStub( "LibQuixote-2.0", true );
if( not quixote ) then
    error( L["%s requires %s"]:format( L["Baldrick"], "LibQuixote-2.0" ) );
    return;
end--if

Baldrick = LibStub( "AceAddon-3.0" ):NewAddon( "Baldrick", "AceEvent-3.0", "AceTimer-3.0", "AceConsole-3.0" );
Baldrick.version = { major = 1, minor = 5, revision = tonumber( "95" ) };

if( LibStub( "LibDataBroker-1.1", true ) ) then
    Baldrick.ldbobj = LibStub( "LibDataBroker-1.1" ):NewDataObject( "Baldrick", {
            type = "data source",
            icon = "Interface\\AddOns\\Baldrick\\Img\\Baldrick",
            label = L["Baldrick"],
            text = L["Nothing to do m'lord..."],
            OnTooltipShow = function( tooltip )
                                if( not tooltip ) then
                                    return;
                                end--if
                                Baldrick:SetTooltip( tooltip );
                            end,
        } );
end--if

Baldrick.DEBUG = false;            --true = Debugging Mode

Baldrick.dprint = LibLordFarlander.dprint;
Baldrick.button = nil;

Baldrick.Dirty = false;
Baldrick.DirtyBags = {};

Baldrick.EventRegistry = {
    --Prevent these from being overridden
    ["Quest_Gained"] = true,
    ["Quest_Failed"] = true,
    ["Quest_Complete"] = true,
    ["Quest_Lost"] = true,
    ["BAG_UPDATE"] = true,
    ["PLAYER_REGEN_ENABLED"] = true,
    ["BAG_UPDATE_COOLDOWN"] = true,
    ["UNIT_INVENTORY_CHANGED"] = true,
};
    
Baldrick.RegisteredCallbackLibraries = {};

Baldrick.QuestModules = {};
Baldrick.AllQuests = {};

function Baldrick:OnInitialize()
    local AceConfigReg = LibStub( "AceConfigRegistry-3.0" );
    local AceConfigDialog = LibStub( "AceConfigDialog-3.0" );
    local defaults = {
        profile = {
            Modules = {},
            Button = LibStub( "LibLordFarlander-UI-2.0" ).GetButtonSettingsDefaults(),
        }
    };
    local buttonOptions = LibStub( "LibLordFarlander-UI-2.0" ).GetButtonOptionGroup( self, {
            name = L["Button"],
            desc = L["Options relating to the button."],
            yes = L["yes"],
            no = L["no"],
            show = L["showing"],
            hide = L["hiding"],
            locked = L["locked"],
            unlocked = L["unlocked"],
            toggleButton = L["toggleButton"],
            toggleButton_name = L["Show button"],
            toggleButton_desc = L["Show/hide the button."],
            autoHideButton = L["autoHideButton"],
            autoHideButton_name = L["Automatically hide button"],
            autoHideButton_desc = L["Toggle the autohiding of the button when there is nothing to do."],
            lockButton = L["lockButton"],
            lockButton_name = L["Lock button position"],
            lockButton_desc = L["Lock/unlock the position of the button."],
            alpha = L["alpha"],
            alpha_name = L["Alpha of button"],
            alpha_desc = L["Sets the alpha value that the button has normally."],
            fadeOut = L["fadeOut"],
            fadeOut_name = L["Fade out"],
            fadeOut_desc = L["Fade the button out when the cursor is not over it?"],
            fadeOutInsteadHide = L["fadeOutInsteadHide"],
            fadeOutInsteadHide_name = L["Fade out instead of hiding"],
            fadeOutInsteadHide_desc = L["Fade the button out instead of hiding it?"],
            fadeOutCancelInCombat = L["fadeOutCancelInCombat"],
            fadeOutCancelInCombat_name = L["Cancel fade out if in combat"],
            fadeOutCancelInCombat_desc = L["Should the button not fade out if you're in combat?"],
            fadeOutCancelOnShift = L["fadeOutCancelOnShift"],
            fadeOutCancelOnShift_name = L["Cancel fade out if holding Shift"],
            fadeOutCancelOnShift_desc = L["Should the button not fade out if you're holding Shift?"],
            fadeOutCancelOnCtrl = L["fadeOutCancelOnCtrl"],
            fadeOutCancelOnCtrl_name = L["Cancel fade out if holding Ctrl"],
            fadeOutCancelOnCtrl_desc = L["Should the button not fade out if you're holding Ctrl?"],
            fadeOutCancelOnAlt = L["fadeOutCancelOnAlt"],
            fadeOutCancelOnAlt_name = L["Cancel fade out if holding Alt"],
            fadeOutCancelOnAlt_desc = L["Should the button not fade out if you're holding Alt?"],
            fadeOutTime = L["fadeOutTime"],
            fadeOutTime_name = L["Fade out time"],
            fadeOutTime_desc = L["Sets how long the button takes to fade out."],
            fadeOutDelay = L["fadeOutDelay"],
            fadeOutDelay_name = L["Fade out delay"],
            fadeOutDelay_desc = L["Sets the delay before the button starts to fade."],
            fadeOutAlpha = L["fadeOutAlpha"];
            fadeOutAlpha_name = L["Alpha after fadeout"],
            fadeOutAlpha_desc = L["Sets the alpha value that the button has after faded out."],
            buttonScale = L["buttonScale"],
            buttonScale_name = L["Scale of button"],
            buttonScale_desc = L["Sets the scaling value for the button to make it bigger or smaller."],            
        }, 1 );
    local options = {
        type = "group",
        handler = self,
        args = {
            modules = {
                type = "multiselect",
                order = 1,
                name = L["Modules"],
                desc = L["Selects which modules to load."],
                width = "full",
                cmdHidden = true,
                values = function( info )
                             local Modules = {};

                             for name, module in pairs( info.handler.QuestModules ) do
                                 if( ( not module.checkLoad ) or module.checkLoad() ) then
                                     Modules[name] = name;
                                 end--if
                             end--for
                             return( Modules );
                         end,
                get = function( info, key )
                          return info.handler.db.profile.Modules[key];
                      end,
                set = function( info, key, val )
                          if( val == nil ) then
                              val = false;
                          end--if
                          info.handler.db.profile.Modules[key] = val;
                          info.handler:RebuildQuestLists();
                      end,
            },
        },
        name = L["Baldrick"],
        desc = L["Addon options."],
        icon = "Interface\\AddOns\\Baldrick\\Img\\Baldrick",
    };

    self.db = LibStub( "AceDB-3.0" ):New( "BaldrickDB", defaults );
    self.db.RegisterCallback( self, "OnProfileChanged" );
    self.db.RegisterCallback( self, "OnProfileReset" );

    local profileOptions = LibStub( "AceDBOptions-3.0" ):GetOptionsTable( self.db );
    local slashOptions = {
        type = "group",
        handler = self,
        name = options.name,
        desc = options.desc,
        icon = options.icon,
        args = {
            [L["button"]] = buttonOptions,
            [L["profile"]] = profileOptions,
            [L["gui"]] = {
                type = "execute",
                order = -1,
                name = L["Open GUI configurator"],
                desc = L["Open GUI configurator"],
                guiHidden = true,
                func = function( info )
                           InterfaceOptionsFrame_OpenToCategory( L[info.handler.name] );
                       end,
            },
            debug = {
                type = "toggle",
                hidden = true,
                name = "Debug",
                desc = "Debug mode toggle",
                desc = function( info )
                            local returnString = "Debug mode toggle";

                            if( info.uiType == "cmd" ) then
                                returnString = ("%s |cFFFF0000[%s]|r"):format( returnString, info.handler.DEBUG and L["yes"] or L["no"] );
                            end--if
                           return returnString;
                       end,
                get = function( info )
                          return info.handler.DEBUG;
                      end,
                set = function( info, val )
                          info.handler.DEBUG = val;
                          info.handler:Print( ("debug mode |cFFFF0000[%s]|r"):format( info.handler.DEBUG and L["yes"] or L["no"] ) );
                      end,
            },
        },
    };

    LibStub( "AceConfig-3.0" ):RegisterOptionsTable( L["Baldrick"], slashOptions, { L["baldrick"], L["br"] } );
    AceConfigReg:RegisterOptionsTable( "Baldrick Main Options", options );
    AceConfigReg:RegisterOptionsTable( "Baldrick Button", buttonOptions );
    AceConfigReg:RegisterOptionsTable( "Baldrick Profile", profileOptions );
    AceConfigDialog:AddToBlizOptions( "Baldrick Main Options", L["Baldrick"] );
    AceConfigDialog:AddToBlizOptions( "Baldrick Button", L["Button"], L["Baldrick"] );
    AceConfigDialog:AddToBlizOptions( "Baldrick Profile", L["Profile"], L["Baldrick"] );
end--Baldrick:OnInitialize()

function Baldrick:AutoHideChange()
    self:SetButton( self:GetItemToShow( self.AllQuests ) );
end--Baldrick:AutoHideChange()

function Baldrick:OnProfileReset()
    self:OnProfileChanged();
end--Baldrick:OnProfileReset()

function Baldrick:OnProfileChanged()
    LibStub( "LibLordFarlander-UI-2.0" ).OnProfileChanged( self );
    self.db.profile.LastWoWVersion = LibLordFarlander.GetTOCVersion();
    self.db.profile.LastVersion = self.version;
    self:BuildQuestLists();
end--Baldrick:OnProfileChanged()

function Baldrick:OnEnable()
    --Register the Events
    quixote.RegisterCallback( self, "Quest_Gained", "Update" );
    quixote.RegisterCallback( self, "Quest_Complete", "Update" );
    quixote.RegisterCallback( self, "Quest_Failed", "Update" );
    quixote.RegisterCallback( self, "Quest_Lost", "Update" );
    quixote.RegisterCallback( self, "Objective_Update", "Update" );
    
    self:RegisterEvent( "BAG_UPDATE",
        function( _, bagID )
            Baldrick.Dirty = true;
            if( bagID ) then
                Baldrick.DirtyBags[bagID] = true;
            end--if
            if( not Baldrick.button:IsVisible() ) then
                Baldrick:ScheduleTimer( "OnUpdate", 0 );
            end--if
        end );
    self:RegisterEvent( "PLAYER_REGEN_ENABLED",
        function()
            Baldrick.button:OutOfCombatReset();
            Baldrick:Update( "PLAYER_REGEN_ENABLED" );
        end );

    self:RegisterEvent( "BAG_UPDATE_COOLDOWN",
        function( ... )
            Baldrick:UpdateCooldown();
            Baldrick:Update( ... );
        end );
    self:RegisterEvent( "UNIT_INVENTORY_CHANGED",
        function( _, unitID )
            if( unitID == "player" ) then
                Baldrick:UpdateCooldown();
                Baldrick:Update( "UNIT_INVENTORY_CHANGED", unitID );
            end--if
        end );

    if( not self.button ) then
        local LibUI = LibStub( "LibLordFarlander-UI-2.0" );

        LibUI.SetupForUI( self );
        self.button = LibUI.CreateButton( self, self.name, "Interface\\AddOns\\Baldrick\\Img\\Baldrick" );
        self.button:SetMovable( not self.db.profile.Button.LockButton );
    end--if

    self.lastItem = nil;

    self:OnProfileChanged();
end--Baldrick:OnEnable()

function Baldrick:OnDisable()
    quixote.UnregisterAll( self );
    for library, _ in pairs( self.RegisteredCallbackLibraries ) do
        LibStub( library ).UnregisterAll( self );
    end--for
    self.RegisteredCallbackLibraries = {};
    self.AllQuests = {};
end--Baldrick:OnDisable()

function Baldrick:OnUpdate( elapsed )
    if( self.Dirty ) then
        --Update the quest item if needed
        self.Dirty = false;
        self:Update( "BAG_UPDATE", self.DirtyBags );
        self.DirtyBags = {};
    end--if
end--Coconuts:OnUpdate()

function Baldrick:Update( event, ... )
    local questList = self.AllQuests;
    local eventRegistery = self.EventRegistry[event];

    if( eventRegistery and ( type( eventRegistry ) == "table" ) and ( #eventRegistery > 0 ) ) then
        questList = eventRegistery;
    end--if
    self:SetButton( self:GetItemToShow( questList, event, ... ) or self.lastItem );
end--Baldrick:Update()

----------------------------
--+Quest Functions--
function Baldrick:RebuildQuestLists()
    -- Set up lists of certain events so that related quests are only checked on those events
    self.AllQuests = {};    
    self.EventRegistry = {
        --Prevent these from being overridden
        ["Quest_Gained"] = true,
        ["Quest_Failed"] = true,
        ["Quest_Complete"] = true,
        ["Quest_Lost"] = true,
        ["Objective_Update"] = true,
        ["BAG_UPDATE"] = true,
        ["PLAYER_REGEN_ENABLED"] = true,
        ["BAG_UPDATE_COOLDOWN"] = true,
        ["UNIT_INVENTORY_CHANGED"] = true,
    };
    self:BuildQuestLists();
end--Baldrick:RebuildQuestLists()

function Baldrick:BuildQuestLists()
    -- Set up lists of certain events so that related quests are only checked on those events
    for name, module in pairs( self.QuestModules ) do
        if( self.db.profile.Modules[name] == nil ) then
            self.db.profile.Modules[name] = true;
        end--if
        if( self.db.profile.Modules[name] and ( ( not module.checkLoad ) or module.checkLoad() ) ) then
            for _, quest in pairs( module.quests ) do
                self:RegisterQuest( quest );
            end--for
        end--if
    end--for
end--Baldrick:BuildQuestLists()

function Baldrick:RegisterQuestModule( moduleName, questList, checkLoadFunc )
    if( not self.QuestModules[moduleName] ) then
        self.QuestModules[moduleName] = { quests = questList, checkLoad = checkLoadFunc };
        if( self.button ) then
            self:BuildQuestLists();
        end--if
    end--if
end--Baldrick:RegisterQuestModule( moduleName, questList, noUseFunc )

function Baldrick:RegisterQuest( quest )
    self:dprint( ("Registering quest %i"):format( ( type( quest.questIDs ) == "table" ) and quest.questIDs[1] or quest.questIDs ) );
    if( quest.subZones ~= nil ) then
        self:RegisterQuestEvent( "ZONE_CHANGED", quest );
        self:RegisterQuestEvent( "ZONE_CHANGED_NEW_AREA", quest );
    end--if
    if( quest.target ~= nil ) then
        self:RegisterQuestEvent( "PLAYER_TARGET_CHANGED", quest );
        self:RegisterQuestEvent( "UNIT_NAME_UPDATE", quest,
            function( event, unitid, ... )
                if( ( unitid == "playertarget" ) or ( unitid == "target" ) ) then
                    Baldrick:Update( event, unitid, ... );
                end--if
            end );
    end--if
    if( quest.additionalEvents ~= nil ) then
        if( type( quest.additionalEvents ) == "table" ) then
            for _, additionalEvent in pairs( quest.additionalEvents ) do
                self:RegisterQuestEvent( additionalEvent, quest );
            end--for
        else
            self:RegisterQuestEvent( quest.additionalEvents, quest );
        end--if
    end--if
    tinsert( self.AllQuests, quest );
end--Baldrick:AddQuest()

function Baldrick:RegisterQuestEvent( event, quest, functionCall )
    local LibraryName = nil;

    if( event:find( ":" ) ) then
        LibraryName, event = strsplit( ":", event );
    end--if
    if( self.EventRegistry[event] == nil ) then
        if( functionCall == nil ) then
            functionCall = function( ... ) Baldrick:Update( ... ); end;
        end--if
        self.EventRegistry[event] = {};
        if( LibraryName ) then
            local Library = LibStub( LibraryName, true );

            if( Library ) then
                Library.RegisterCallback( self, event, functionCall );
                self.RegisteredCallbackLibraries[LibraryName] = true;
            end--if
        else
            self:RegisterEvent( event, functionCall );
        end--if
    end--if
    if( type( self.EventRegistry[event] ) == "table" ) then
        table.insert( self.EventRegistry[event], quest );
    end--if
end--Baldrick:RegisterQuest( event, quest )

function Baldrick.CheckQuestIDs( questIDs )
    if( type( questIDs ) == "number" ) then
        Baldrick:dprint( "Checking " .. questIDs );
        if( Baldrick.HasQuestAndNotComplete( questIDs ) ) then
            Baldrick:dprint( "Returning " .. questIDs );
            return questIDs;
        end--if
    else
        for _, questID in pairs( questIDs ) do
            Baldrick:dprint( "Checking " ..  questID );
            if( Baldrick.HasQuestAndNotComplete( questID ) ) then
                Baldrick:dprint( "Returning " .. questID );
                return questID;
            end--if
        end--for
    end--if
    return nil;
end--DailyQuest.CheckQuestIDs( questIDs )

function Baldrick.CheckItems( questItems )
    local bItems = true;

    for _, item in pairs( questItems ) do
        bItems = ( LibLordFarlander.PlayerItemCount( item.questItem ) >= item.questItemCount ) and bItems;
    end--for
    return bItems;
end--Baldrick.CheckItems( questItems )

function Baldrick:GetItemToShow( questList, event, ... )
    local returnVal = nil;
    local playerLevel = UnitLevel( "player" );

    if( questList ) then
        local subZone = GetSubZoneText();
        local zone = GetRealZoneText();
        local target, _ = UnitName( "playertarget" );

        for _, quest in pairs( questList ) do
            if( ( not quest.levelMin ) or ( quest.levelMin <= playerLevel ) ) then
                local questTarget = quest.target;
                local questSubZones = quest.subZones;
                local questFunc = quest.func;
                local questID = Baldrick.CheckQuestIDs( quest.questIDs );
                local eventFunction = event and quest[("On%s"):format( event )];

                if( ( returnVal == nil ) and ( questID ~= nil ) and
                    Baldrick.CheckItems( quest.questItems ) and
                    ( ( questTarget == nil ) or ( target and ( questTarget:find( target ) ) ) ) and
                    ( ( questSubZones == nil ) or ( ( subZone ~= "" ) and ( questSubZones:find( subZone ) ~= nil ) ) or ( ( subZone == "" ) and ( zone ~= "" ) and ( questSubZones:find( GetRealZoneText() ) ~= nil ) ) ) and
                    ( ( questFunc == nil ) or questFunc( quest, questID ) )
                    ) then
                    returnVal = quest.questItems.default.questItem;
                    Baldrick:dprint( ("return item %i"):format( returnVal ) );
                elseif( quest.questItems.default.questItem == self.lastItem ) then
                    self.lastItem = nil;
                end--if
                if( eventFunction ) then
                    if( type( eventFunction ) == "string" ) then
                        quest[eventFunction]( quest, ... );
                    else
                        eventFunction( quest, ... );
                    end--if
                end--if
            end--if
        end--for
    end--if
    return returnVal or self.lastItem;
end--Baldrick:GetItemToShow( questList, event, ... )
---Quest Functions--
----------------------------

-------------------
--+Help Functions--
function Baldrick.HasQuestAndNotComplete( quest )
    local uid, _, _, _, _, _, complete, _, _, _ = quixote:GetQuestByUid( quest );

    return ( uid ~= nil) and ( not complete );
end--self.HasQuestAndNotComplete( quest )

function Baldrick.QuestObjectiveComplete( questId, objective )
    if( type( questId ) ~= "number" ) then
        questId, _, _, _, _, _, _, _, _, _  = quixote:GetQuest( questId );
    end--if
    if( questId ~= nil ) then
        local _, numItems, numNeeded = quixote:GetQuestObjective( questId, objective );

        return numItems == numNeeded;
    end--if
    return false;
end--Baldrick.QuestObjectiveComplete( quest, objective )

function Baldrick.GetQuestName( questID )
    return select( 3, quixote:GetQuestByUid( quest.questIDs ) );
end--Baldrick.GetQuestName( questID )
---Help Functions--
-------------------

-----------------
--+UI Functions--
function Baldrick:SetTooltip( tooltip )
    if( self.lastItem == nil ) then
        tooltip:SetText( L["Nothing to do m'lord..."] );
    else
        tooltip:SetHyperlink( LibLordFarlander.GetItemLink( self.lastItem ) );
    end--if
end--Baldrick:SetTooltip( tooltip )

function Baldrick:SetGameTooltips()
    local dqButton = self.button;

    if( not self.keyBoundMode ) then
        local dqTooltip = dqButton.tooltip;

        dqTooltip:SetOwner( self.button, "RIGHT" );
        self:SetTooltip( dqTooltip );
    end--if
    LibStub( "LibLordFarlander-UI-2.0" ).AfterSetTooltip( dqButton );
end--Baldrick:SetGameTooltips()

function Baldrick:UpdateCooldown()
    local itemLink = self.lastItem;

    if( itemLink ) then
        local startTime, duration, enable = GetItemCooldown( itemLink );
        local cooldownInicator = self.button.cooldown;
        
        if( startTime and duration and enable and ( startTime > 0 ) and ( duration > 0 ) ) then
            cooldownInicator:Show();
            cooldownInicator:SetCooldown( startTime, duration );
        else
            cooldownInicator:Hide();
        end--if
    end--if
end--Baldrick:UpdateCooldown()

function Baldrick:SetButton( questItem )
    if( not InCombatLockdown() ) then
        local dqButton = self.button;
        local cooldownInicator = dqButton.cooldown;
        local lastitem = self.lastItem;
        local chardb = self.db.profile;
        local ldbobj = self.ldbobj;

        -- ONLY change if not in combat
        if( questItem == nil ) then
            cooldownInicator:Hide();
            dqButton:SetAttribute( "*type1", "item" );
            dqButton:SetAttribute( "*item1", nil );
    
            dqButton:SetImage( "Interface\\AddOns\\Baldrick\\Img\\Baldrick" );

            ldbobj.icon = "Interface\\AddOns\\Baldrick\\Img\\Baldrick";
            ldbobj.text = L["Nothing to do m'lord..."];
            if( chardb.Button.AutoHideButton ) then
                dqButton:HideEx();
            end--if
        else
            local sName, _, _, _, _, _, _, _, _, sItemTexture = GetItemInfo( questItem );
            local startTime, duration, enable = GetItemCooldown( questItem );

            dqButton:SetAttribute( "*type1", "item" );
            dqButton:SetAttribute( "*item1", sName );

            dqButton:SetImage( sItemTexture );

            ldbobj.icon = sItemTexture;
            ldbobj.text = sName;

            dqButton:Enable();
            if( chardb.Button.AutoHideButton ) then
                dqButton:ShowEx();
            end--if            
            if( ( lastitem ~= questItem ) and startTime and duration and enable and ( startTime > 0 ) and ( duration > 0 ) ) then
                cooldownInicator:Show();
                cooldownInicator:SetCooldown( startTime, duration );
            end--if
        end--if
    end--if
    self.lastItem = questItem;
end--Baldrick:SetButton( questItem )
---UI Functions--
-----------------
